#include "inflow.h"
#include <string>
#include "convertstring.h"
#include <boost/random.hpp>
#include <boost/random/normal_distribution.hpp>
#include "slave.h"

Inflow::Inflow(int in_num_stages, int in_num_modules, int in_num_samples, int in_num_branches, int in_max_num_iterations)
	:num_stages(in_num_stages), num_modules(in_num_modules), num_samples(in_num_samples), num_branches(in_num_branches), max_num_iterations(in_max_num_iterations) {
	inflow_data = new double*[52];
	sxx = new double[52];
	sxy = new double[52];
	correlation = new double[52];

	for(int i=0; i < 52; i++){
		inflow_data[i] = new double[30];
		std::fill_n(inflow_data[i], 30, 0.0);
	}
	std::fill_n(sxx , 52, 0.0);
	std::fill_n(sxy , 52, 0.0);
	std::fill_n(correlation , 52, 0.0);
}

void Inflow::Initialize(std::ifstream& dsrc) {
	double current_value;
	std::string token;
	std::string temp;
	size_t pos ;
	size_t new_pos;
	size_t length; 
//	random_number_generator.seed(time(NULL));

	dsrc >> token; //skip years.
	for (int i = 0; i < 52; i++){
		dsrc >> token;
		pos = 0;
		new_pos = 0;
		for (int j = 0; j < 30; j++){
			new_pos = token.find(';', pos);
			length = new_pos - pos;
			temp = token.substr(pos, length);
			current_value = convertString<double>(temp);
			inflow_data[i][j] = current_value;
			pos = new_pos + 1;
		}
	}
	dsrc.close();

	CalcMean();
	yearly_mean = 0;
	for(int i = 0; i < 52; i++) {
		yearly_mean += mean[i];
	}
	makeInflowRelative();

	CalcVar();
	CalcStdDev();
	Normalize();
	CalcCorrelation();
	CalcCorrelatedVar();
	CalcCorrelatedStdDev();
	ModifyCorrelation();

	noise_samples = new double**[max_num_iterations];
	for (int i = 0; i < max_num_iterations; i++) {
		noise_samples[i] = new double*[num_stages];
		for (int j = 0; j < num_stages; j ++) {
			noise_samples[i][j] = new double[num_samples];
			std::fill_n(noise_samples[i][j], num_samples, 0.0);
		}
	}

	forward_inflow = new double**[max_num_iterations];
	for (int i = 0; i < max_num_iterations; i++) {
		forward_inflow[i] = new double*[num_stages];
		for (int j = 0; j < num_stages; j++) {
			forward_inflow[i][j] = new double[num_branches];
			std::fill_n(forward_inflow[i][j], num_branches, 0.0);
		}
	}
}

void Inflow::SeedRNG(int seed){
	random_number_generator.seed( static_cast<unsigned int>(seed) );
}

void Inflow::CalcMean(){
	mean = new double[52];
	std::fill_n(mean, 52, 0.0);

	for (int i = 0; i < 52; i++){
		for (int j = 0; j < 30; j++){
			mean[i] += inflow_data[i][j];
		}
		mean[i] = mean[i]/30;
	}
}

void Inflow::makeInflowRelative(){
	for (int i = 0; i < 52; i++){
		for (int j = 0; j < 30; j++){
			inflow_data[i][j] = inflow_data[i][j]/yearly_mean;
		}
		mean[i] = mean[i]/yearly_mean;
	}
}

void Inflow::CalcVar(){
	var = new double[52];

	for (int i = 0; i < 52; i++){
		for (int j = 0; j < 30; j++){
			var[i] += (mean[i] - inflow_data[i][j])*(mean[i] - inflow_data[i][j]);
		}
		var[i] = var[i]/(30-1);
	}
}

void Inflow::CalcStdDev(){
	std_dev = new double[52];
	for (int i = 0; i < 52; i++){
		std_dev[i] = std::sqrt(var[i]);
	}
}

void Inflow::Normalize(){

	normalized_data = new double*[52];

	for(int i=0; i < 52; i++){
		normalized_data[i] = new double[30];
		std::fill_n(normalized_data[i], 30, 0);
	}

	for (int i = 0; i < 52; i++){
		for (int j = 0; j < 30; j++){
			normalized_data[i][j] = (inflow_data[i][j] - mean[i])/std_dev[i];
		}
	}
}

void Inflow::CalcCorrelation(){
	for (int i = 0; i < 51; i++){
		for (int j = 0; j < 30; j++){
			sxx[i] += normalized_data[i][j]*normalized_data[i][j];
			sxy[i] += normalized_data[i + 1][j]*normalized_data[i][j];
		}
//		correlation[i + 1] = sxy[i]/sxx[i];
		correlation[i + 1] = 0;
	}

	for (int j = 0; j < 29; j++){
			sxx[51] += normalized_data[51][j]*normalized_data[51][j];
			sxy[51] += normalized_data[0][j+1]*normalized_data[51][j];
	}
//	correlation[0] = sxy[51]/sxx[51];
	correlation[0] = 0;
}

double*** Inflow::CreateNoiseSamples() {
	double*** new_noise_samples = new double**[max_num_iterations];
	for (int i = 0; i < max_num_iterations; i++) {
		new_noise_samples[i] = new double*[num_stages];
		for (int j = 0; j < num_stages; j++) {
			new_noise_samples[i][j] = new double[num_samples];
			for (int k = 0; k < num_samples; k++) {
				new_noise_samples[i][j][k] = create_noise(i);
			}
		}
	}
	return new_noise_samples;
}

double*** Inflow::CreateForwardInflow() {
	double*** new_forward_inflow = new double**[max_num_iterations];
	for (int i = 0; i < max_num_iterations; i++) {
		new_forward_inflow[i] = new double*[num_stages];
		for (int j = 0; j < num_stages; j++) {
			new_forward_inflow[i][j] = new double[num_branches];
			for (int k = 0; k < num_branches; k++) {
				if (j == 0) {
					new_forward_inflow[i][j][k] = create_inflow(j, 0);
				}
				else {
					new_forward_inflow[i][j][k] = create_inflow(j, forward_inflow[i][j][k - 1]);
				}
			}
		}
	}
	return new_forward_inflow;
}

void Inflow::Print(){
	log.open("../../output/inflow.txt");
	for (int i = 0; i < 51; i++){
		log << correlation[i] << ";";
	}
	log << correlation[51] << std::endl;

	for (int i = 0; i < 51; i++) {
		log << mean[i] << ";";
	}
	log << mean[51] << std::endl;

	for (int i = 0; i < 51; i++) {
		log << get_std_dev(i) << ";";
	}
	log << get_std_dev(51) << std::endl;

	for (int i = 0; i < 30; i++){
		for (int j = 0; j < 52; j++) {
			log << inflow_data[j][i] << ";";
		}
		log << std::endl;
	}

	for (int i = 0; i < 30; i++){
		for (int j = 0; j < 52; j++) {
			log << normalized_data[j][i] << ";";
		}
		log << std::endl;
	}
	for (int i = 0; i < 1; i++) {
		for (int j = 0; j < num_stages; j++) {
			for (int k = 0; k < num_samples; k++) {
				log << noise_samples[i][j][k] << ";";
			}
			log << mean[i] << ";";
			log << std::endl;
		}
	}

	for (int i = 0; i < max_num_iterations; i++) {
		for (int j = 0; j < num_stages; j++) {
			for (int k = 0; k < num_branches; k++) {
				log << forward_inflow[i][j][k] << ";";
			}
			log << std::endl;
		}
	}

	log.close();
}

double Inflow::create_inflow(int stage, double previous_inflow){
	if (stage == 0)
		return mean[0];
	else{
		int previous_stage = (stage - 1)%52;
		double return_value;
		stage = stage%52;
		if ( get_std_dev(previous_stage) == 0.0 ) {
			return_value = mean[stage] + create_noise(stage);
			std::cout << "Would have deleted by 0!" << std::endl;
		}
		else {
			//return_value = correlation[stage]*(previous_inflow - mean[previous_stage])*get_std_dev(stage)/get_std_dev(previous_stage) + mean[stage] + create_noise(stage);
			//return_value = correlation[stage]*(previous_inflow - mean[previous_stage]) + mean[stage] - std::abs( create_noise(stage));
			//return_value = correlation[stage]*(previous_inflow - mean[previous_stage]) + mean[stage];
			return_value = correlation[stage]*(previous_inflow - mean[previous_stage]) + mean[stage] + create_noise(stage);
		}
		return return_value;
	}
//	return create_non_negative_inflow(stage, previous_inflow);
}

double Inflow::create_non_negative_inflow(int stage, double previous_inflow){
	if (stage == 0)
		return mean[0];
	else{
		int previous_stage = (stage - 1)%52;
		double return_value;
		stage = stage%52;

		if ( get_std_dev(previous_stage) == 0.0 ) {
			return_value = mean[stage] + create_noise(stage);
			std::cout << "Would have deleted by 0!" << std::endl;
		}
		else {
			//return_value = correlation[stage]*(previous_inflow - mean[previous_stage])*get_std_dev(stage)/get_std_dev(previous_stage) + mean[stage] + create_noise(stage);
			//return_value = correlation[stage]*(previous_inflow - mean[previous_stage]) + mean[stage] - std::abs( create_noise(stage));
			//return_value = correlation[stage]*(previous_inflow - mean[previous_stage]) + mean[stage];
			return_value = correlation[stage]*(previous_inflow - mean[previous_stage]) + mean[stage] + create_noise(stage);
		}

		if ( return_value < 0 ) {
			return_value = 0;
		}
		return return_value;
	}
}

double Inflow::get_inflow_sample(int stage, int sample, double previous_inflow, int iter){
	if (stage == 0){
		return mean[0];
	}
	else{
		double return_value;
		if ( get_std_dev( (stage - 1)%52 ) == 0.0 ) {
			return_value = mean[stage%52] + get_noise_sample(stage, sample, iter);
			std::cout << "Would have deleted by 0!" << std::endl;
		}
		else {
//			return_value = correlation[stage%52]*(previous_inflow - mean[(stage - 1)%52])*get_std_dev(stage%52)/get_std_dev( (stage - 1)%52 ) + mean[stage%52] + get_noise_sample(stage, sample);
			return_value = correlation[stage%52]*(previous_inflow - mean[(stage - 1)%52]) + mean[stage%52] + get_noise_sample(stage, sample, iter);
		}


		return return_value;
	}
//	return get_non_negative_inflow_sample(stage, sample, previous_inflow);
}

double Inflow::get_non_negative_inflow_sample(int stage, int sample, double previous_inflow, int iter){
	if (stage == 0){
		return mean[0];
	}
	else{
		double return_value;

		if ( get_std_dev( (stage - 1)%52 ) == 0.0 ) {
			return_value = mean[stage%52] + get_noise_sample(stage, sample, iter);
			std::cout << "Would have deleted by 0!" << std::endl;
		}
		else {
//			return_value = correlation[stage%52]*(previous_inflow - mean[(stage - 1)%52])*get_std_dev(stage%52)/get_std_dev( (stage - 1)%52 ) + mean[stage%52] + get_noise_sample(stage, sample);
			return_value = correlation[stage%52]*(previous_inflow - mean[(stage - 1)%52]) + mean[stage%52] + get_noise_sample(stage, sample, iter);
		}
		if ( return_value < 0 ) {
			return_value = 0;
		}
		return return_value;
	}
}

double Inflow::create_noise(int stage){
	double ret;
	//boost::normal_distribution<double> distribution(0, 0);
	boost::normal_distribution<double> distribution(0, get_std_dev(stage));
	boost::variate_generator<boost::mt19937 &, boost::normal_distribution<> > generator(random_number_generator, distribution);
	ret = generator();
//	if (ret < 0.0) {
//		ret = 0.0;
//	}
	return ret;
}

double Inflow::get_noise_sample(int stage, int sample, int iter) {
	return noise_samples[0][stage][sample];
}

void Inflow::ResetNoiseSamples(double*** in_noise_samples) {
	for (int i = 0; i < max_num_iterations; i++) {
		for (int j = 0; j < num_stages; j++) {
			delete[] noise_samples[i][j];
		}
		delete[] noise_samples[i];
	}
	delete[] noise_samples;
	noise_samples = in_noise_samples;
}

void Inflow::ResetForwardInflow(double*** in_forward_inflow) {
	for (int i = 0; i < max_num_iterations; i++) {
		for (int j = 0; j < num_stages; j++) {
			delete[] forward_inflow[i][j];
//			forward_inflow[i][j] = in_forward_inflow[i][j];
		}
		delete[] forward_inflow[i];
	}
	delete[] forward_inflow;
	forward_inflow = in_forward_inflow;
}

double Inflow::get_forward_inflow(int iteration, int stage, int branch) {
	return forward_inflow[iteration][stage][branch];
}

double Inflow::get_previous_forward_inflow(int iteration, int stage, int branch) {
	if (stage == 0) {
		return 0;
	}
	else {
		return forward_inflow[iteration][stage - 1][branch];
	}
}

void Inflow::CalcCorrelatedVar() {
	double temp_var = 0.0;
	correlated_var = new double[52];
	for (int i = 1; i < 53; i++) {
		temp_var = 0.0;
		for (int j = 0; j < 30; j++) {
			temp_var += ((inflow_data[i-1][j] - mean[i-1])*correlation[i%52] + mean[i%52] - inflow_data[i%52][j])*((inflow_data[i-1][j] - mean[i-1])*correlation[i%52] + mean[i%52] - inflow_data[i%52][j]);
		}
		correlated_var[i%52] = temp_var/29;
	}
}

void Inflow::CalcCorrelatedStdDev() {
	correlated_std_dev = new double[52];
	for (int i = 0; i < 52; i++) {
		correlated_std_dev[i] = std::sqrt(get_correlated_var(i));
		std_dev[i] = correlated_std_dev[i]/2;
	}
}

void Inflow::ModifyCorrelation() {
	for (int i = 0; i < 51; i++) {
//		correlation[i + 1] = correlation[i + 1]*get_std_dev(i+1)/get_std_dev(i);
		correlation[i + 1] = 0;
	}
//	correlation[0] = correlation[0]*get_std_dev(0)/get_std_dev(51);
	correlation[0] = 0;
}