#ifndef INFLOW_H
#define INFLOW_H
#include <iostream>
#include <fstream>
#include <sstream>
#include <vector>
#include <cmath>
#include <boost/random.hpp>
#include <boost/random/normal_distribution.hpp>

//typedef double** InflowDatatype;

class Inflow
{
   private:
	  int num_stages;
	  int num_modules;
	  int num_samples;
	  int num_branches;
	  int max_num_iterations;
	  double yearly_mean;
	  double* mean;
	  double* var;
	  double* std_dev;
	  double* correlation;
	  double* correlated_var;
	  double* correlated_std_dev;
	  double* sxx;
	  double* sxy;
	  double*** noise_samples;
	  double*** forward_inflow;
      double** inflow_data;
	  double** normalized_data;
	  boost::mt19937 random_number_generator;
	  std::ofstream log;
	  //correlation, mean, variance, 30 normalized, num_stages of noise_samples + mean, each inflow_samples consists of num_samples.

	  void CalcMean();
	  void makeInflowRelative();
	  void CalcVar();
	  void CalcStdDev();
	  void Normalize();
	  void CalcCorrelation();
	  void CalcCorrelatedVar();
	  void CalcCorrelatedStdDev();
	  void ModifyCorrelation();
	  double create_noise(int stage);
	  double create_inflow(int stage, double previous_inflow);
	  double create_non_negative_inflow(int stage, double previous_inflow);
   public:
      Inflow(int in_num_stages, int in_num_modules, int in_num_samples, int in_num_branches, int in_max_num_iterations);
	  void Initialize(std::ifstream& inflow_data);
	  void SeedRNG(int seed);
	  void Print();
	  double*** CreateNoiseSamples();
	  double*** CreateForwardInflow();
      int get_num_stages() const {return num_stages;}
	  int get_num_modules() const {return num_modules;}
	  double get_mean(int i) {return mean[i%52];}
	  double get_var(int i) {return var[i%52];}
//	  double get_var(int i) {return correlated_var[i%52];}
	  double get_correlated_var(int i){return correlated_var[i%52];}
	  double get_std_dev(int i) {return std_dev[i%52]; }
	  double get_correlated_std_dev(int i) {return correlated_std_dev[i];}
	  double get_correlation(int i) {return correlation[i%52];}
	  double get_inflow_sample(int stage, int sample, double previous_inflow, int iter);
	  double get_non_negative_inflow_sample(int stage, int sample, double previous_inflow, int iter);
	  double get_noise_sample(int stage, int sample, int iter);
	  double*** get_noise_samples(){return noise_samples; }
	  void set_noise_samples(double*** in_noise_samples){noise_samples = in_noise_samples;}
	  void set_forward_inflow(double*** in_forward_inflow){forward_inflow = in_forward_inflow;}
	  double*** get_forward_inflow(){return forward_inflow;}
	  double get_forward_inflow(int iteration, int stage, int branch);
	  double get_previous_forward_inflow(int iteration, int stage, int branch);
	  void ResetNoiseSamples(double*** in_noise_samples);
	  void ResetForwardInflow(double*** in_forward_inflow);
};
#endif