#include <iostream>
#include <fstream>
#include <math.h>
#include "convertstring.h"
#include "powersystem.h"
#include "slave.h"
#include "hydromodule.h"
#include "reservoir.h"
#include "mpi.h"
#include "master.h"
#include <time.h>
#include "inflow.h"
#include "input.h"
#include "mysimplexsolvercoinclp.h"

PowerSystem* CreatePowerSystem(double);
Inflow* CreateInflowData(int, Input*, int rank);
void PrintProblemData(Input*);

int main(int argc, char* argv[]){
	int rank;
	int size;

	//clock_t begin, end;
	std::ofstream results_log;
	MPI_Init(&argc, &argv);
	MPI_Comm_rank(MPI_COMM_WORLD, &rank);
	MPI_Comm_size(MPI_COMM_WORLD, &size);

	Input* input = new Input();
	if (argc == 1) {
		std::ifstream sddp_data("../../data/sddp");
		input->Initialize(sddp_data);
	}
	else {
		input->Initialize(argv);
	}

	PowerSystem* power_system = CreatePowerSystem(input->initial_reservoir);
	int num_modules = int(power_system->getHydroModules().size());
	Inflow* inflow_data = CreateInflowData(num_modules, input, rank);

	if (rank == 0) {
		std::cout << size << " processors" << std::endl;
		PrintProblemData(input);
		//begin = clock();
		if (input->should_print) {
			power_system->printThermalSystem();
			power_system->printHydroSystem();
		}
	}

	if (rank == 0){
		Master* master = new Master(power_system, rank, size);
		master->Initialize(input, inflow_data);
		master->RunSimulations();
		master->Finalize();
	}
	else{
		Slave* slave = new Slave(power_system, rank, size);
		slave->Initialize(input, inflow_data);
		slave->RunSimulation();
		slave->Finalize();
	}

	if (rank == 0) {
		//end = clock();
		//double diff = double(end - begin)/CLOCKS_PER_SEC;
		//std::cout << "solved in " << diff << " seconds." << std::endl;
		/*
		results_log.open("../../output/results.txt", std::ofstream::app);
		results_log << size << ";" << diff << ";" << std::endl;
		results_log.close();
		*/
	}
	MPI_Finalize();

	
	
}

PowerSystem* CreatePowerSystem(double initial_reservoir){
	std::ifstream ctrl_data("../../data/ctrldata",std::ios_base::in);
	std::string ctrl_string;
	ctrl_data >> ctrl_string;
	std::ifstream hydro_data(ctrl_string.c_str(),std::ios_base::in);
	ctrl_data >> ctrl_string;
	std::ifstream thermal_data(ctrl_string.c_str(),std::ios_base::in);
	PowerSystem* const ps = new PowerSystem(hydro_data,thermal_data, initial_reservoir);
	return ps;
}

Inflow* CreateInflowData(int num_modules, Input* input, int rank){
	Inflow* inflow_data = new Inflow(input->num_stages, num_modules, input->num_samples, input->num_branches, input->max_num_iterations);
	std::ifstream stream("../../data/tilsig");
	inflow_data->Initialize(stream);
	return inflow_data;
}

void PrintProblemData(Input* input){
	input->Print();
}