#include "mysimplexsolver2.h"
#include <iostream>
#include <fstream>
#include <sstream>
#include <string>

MySimplexSolver2::MySimplexSolver2() {
	lp_model = make_lp(0, 0);
	set_epsb(lp_model, 1e-5);
	set_epsd(lp_model, 1e-5);
	log_num = 0;
}

void MySimplexSolver2::Reset() {
	delete_lp(lp_model);
}

void MySimplexSolver2::Reset(int num_variables) {
	Reset();
}

void MySimplexSolver2::InitialSolve() {
	solution_error = solve(lp_model);
	switch(solution_error) {
	case -2:
		std::cout << "out of memory" << std::endl;
		break;
	case 0:
//		std::cout << "solved!" << std::endl;
		break;
	case 1:
		std::cout << "suboptimal" << std::endl;
		break;
	case 2:
		std::cout << "infeasible" << std::endl;
		break;
	case 3:
		std::cout << "unbounded" << std::endl;
		break;
	case 4:
		std::cout << "degenerate" << std::endl;
		break;
	case 5:
		std::cout << "numerical failure" << std::endl;
		break;
	case 6:
		std::cout << "user aborted" << std::endl;
		break;
	case 7:
		std::cout << "time out" << std::endl;
		break;
	case 9:
		std::cout << "could be solved by presolve" << std::endl;
		break;
	default:
		std::cout << "other" << std::endl;
	}

	if (solution_error != 0) {
		std::stringstream temp_string;
		temp_string << "../../output/lp_prints/lp";
		temp_string << log_num;
		std::string s = temp_string.str();
		const char* file = s.c_str();
		write_lp(lp_model, const_cast<char*>(file));
		log_num++;
	}
}

void MySimplexSolver2::Resolve() {
	InitialSolve();
}

void MySimplexSolver2::LoadProblem(int num_variables, std::vector< std::map<int, double> >& constraints, double* col_lb, double* col_ub, 
								  double* objective_coefficients, double* row_lb, double* row_ub) {

	int* current_variables;
	double* current_coefficients;
	std::map<int, double> current_constraint;
	int current_num_variables;
	int counter;

	lp_model = make_lp(0, num_variables);
	set_epsb(lp_model, 1e-5);
	set_epsd(lp_model, 1e-5);

	LoadObjectiveFunction(objective_coefficients, num_variables);
	LoadVariableBounds(num_variables, col_lb, col_ub);

	set_add_rowmode(lp_model, TRUE);

	for (int i = 0; i < constraints.size(); i ++) {
		current_constraint = constraints[i];
		current_num_variables = current_constraint.size();
		current_variables = new int[current_num_variables];
		current_coefficients = new double[current_num_variables];

		counter = 0;
		for(std::map<int,double>::iterator iter = current_constraint.begin(); iter != current_constraint.end(); ++iter) {
			current_variables[counter] = iter->first + 1;
			current_coefficients[counter] = iter->second;
			counter++;
		}
		if (row_ub[i] == row_lb[i]) {
			add_constraintex(lp_model, current_num_variables, current_coefficients, current_variables, EQ, row_ub[i]);
		}
		else {
			add_constraintex(lp_model, current_num_variables, current_coefficients, current_variables, GE, row_lb[i]);
		}

		delete[] current_variables;
		delete[] current_coefficients;
	}
	set_add_rowmode(lp_model, FALSE);
	set_minim(lp_model);
	set_verbose(lp_model, NEUTRAL);

}

void MySimplexSolver2::LoadObjectiveFunction(double* objective_coefficients, int num_variables){
//	objective_coefficients--;
	double* temp = new double[num_variables + 1];
	std::copy(objective_coefficients, objective_coefficients + num_variables, temp + 1);
	temp[0] = 0;
//	double* temp2 = temp++;
//	temp2 = objective_coefficients;
	set_obj_fn(lp_model, temp);
	delete[] temp;
}

void MySimplexSolver2::LoadVariableBounds(int num_variables, double* col_lb, double* col_ub) {
	for (int i = 0; i < num_variables; i++) {
		set_bounds(lp_model, i + 1, col_lb[i], col_ub[i]);
	}
}

void MySimplexSolver2::SetRowBounds(int constraint, double lower_bound, double upper_bound) {
//	bool temp;
//	temp = set_rh(lp_model, constraint + 1, lower_bound);
//	std::cout << temp << " " << constraint << " " << lower_bound << std::endl;
	set_rh(lp_model, constraint + 1, lower_bound);
}

double* MySimplexSolver2::GetColSolution(){
	double** col_solution = new double*;
	get_ptr_variables(lp_model, col_solution);
	double* ret = *col_solution;
	delete col_solution;
	return ret;
}

double* MySimplexSolver2::GetRowPrice(){
	double** dual_solution = new double*;
	get_ptr_dual_solution(lp_model, dual_solution);
	double* ret = *dual_solution + 1;
	delete dual_solution;
	return ret;
}

void MySimplexSolver2::WriteLP(const char* file){
	write_mps(lp_model, const_cast<char*>(file));
}

void MySimplexSolver2::PrintSolution(const char* file) {
	set_outputfile(lp_model, const_cast<char*>(file));
	print_solution(lp_model, get_Ncolumns(lp_model));
}