#include "powersystem.h"
#include "powerplant.h"
#include "reservoir.h"
#include "hydromodule.h"
#include "thermalplant.h"
#include "convertstring.h"
#include <string>

PowerSystem::PowerSystem(std::ifstream& hydroData,std::ifstream& thermalData)
{
   bool stop = establishHydroSystem(hydroData);
   //stop = establishThermalSystemFromFile(thermalData);
   establishThermalSystem();
}

PowerSystem::PowerSystem(std::ifstream& hydroData,std::ifstream& thermalData, double initial_reservoir)
{
   bool stop = establishHydroSystem(hydroData, initial_reservoir);
   //stop = establishThermalSystemFromFile(thermalData);
   establishThermalSystem();
}

bool PowerSystem::establishHydroSystem(std::ifstream& dsrc)
{
   bool stop = false;
   if (dsrc.fail()) {
      std::cout << "Unable to open data file for reading..."<<std::endl;
      stop = true;
   }
   std::string token;
   bool plant = true;
   bool reservoir = true;
   bool connectivity = true;
//   bool demand = true;
   int cnt = 0;

   while (dsrc >> token && stop == false) {
      if (token == "#") {
         dsrc.ignore(80,'\n');
      }
      else if (plant) {
         while (dsrc >> token && token != "#") {
            int mNr = convertString<int>(token);
            dsrc >> token;
            double cap = convertString<double>(token);
            dsrc >> token;
            double eff = convertString<double>(token);
            PowerPlant* pp = new PowerPlant(mNr,cap,eff);
            PowerPlants.push_back(pp);
            cnt++;
         }
         if (token == "#") {
            dsrc.ignore(80,'\n');
         }
         cnt = 0;
         plant = false;
      }
      else if (reservoir) {
         while (dsrc >> token && token != "#") {
            int mNr = convertString<int>(token);
            dsrc >> token;
            double cap = convertString<double>(token);
            dsrc >> token;
            double init = convertString<double>(token);
			dsrc >> token;
            double yearly_inflow = convertString<double>(token);
            Reservoir* hr = new Reservoir(mNr,cap,init, yearly_inflow);
            Reservoirs.push_back(hr);
            cnt++;
         }
         if (token == "#") {
            dsrc.ignore(80,'\n');
         }
         cnt = 0;
         reservoir = false;

         for (size_t i=0; i<Reservoirs.size(); i++) {
            int mNr1 = Reservoirs[i]->getModuleNr();
            for (size_t j=0; j<PowerPlants.size(); j++)
            {
               int mNr2 = PowerPlants[j]->getModuleNr();
               if (mNr1 == mNr2) {
                  cnt++;
                  HydroModule* hm = new HydroModule(cnt,*(Reservoirs[i]),*(PowerPlants[j]));
                  HydroModules.push_back(hm);
                  break;
               }
            }
         }
         cnt = 0;
      }
      else if (connectivity) {
         while (dsrc >> token && token != "#") {
            int from = convertString<int>(token)-1;
            dsrc >> token;
            int to = convertString<int>(token)-1;
            HydroModules[from]->setDownStreamModule(HydroModules[to]);
            HydroModules[to]->setUpStreamModules(HydroModules[from]);
            cnt++;
         }
         if (token == "#") {
            dsrc.ignore(80,'\n');
         }
         connectivity = false;
         cnt = 0;
      }
      //else if (demand) {
      //   while (dsrc >> token && token != "#") {
      //      double dm = convertString<double>(token);
      //      Demand* dmnd = new Demand(dm);
      //      FirmDemand = dmnd;
      //      cnt++;
      //   }
      //   if (token == "#") {
      //      dsrc.ignore(80,'\n');
      //   }
      //   demand = false;
      //   cnt = 0;
      //}
   }
   dsrc.close();
   return stop;
}

bool PowerSystem::establishHydroSystem(std::ifstream& dsrc, double initial_reservoir)
{
   bool stop = false;
   if (dsrc.fail()) {
      std::cout << "Unable to open data file for reading..."<<std::endl;
      stop = true;
   }
   std::string token;
   bool plant = true;
   bool reservoir = true;
   bool connectivity = true;
//   bool demand = true;
   int cnt = 0;

   while (dsrc >> token && stop == false) {
      if (token == "#") {
         dsrc.ignore(80,'\n');
      }
      else if (plant) {
         while (dsrc >> token && token != "#") {
            int mNr = convertString<int>(token);
            dsrc >> token;
            double cap = convertString<double>(token);
            dsrc >> token;
            double eff = convertString<double>(token);
            PowerPlant* pp = new PowerPlant(mNr,cap,eff);
            PowerPlants.push_back(pp);
            cnt++;
         }
         if (token == "#") {
            dsrc.ignore(80,'\n');
         }
         cnt = 0;
         plant = false;
      }
      else if (reservoir) {
         while (dsrc >> token && token != "#") {
            int mNr = convertString<int>(token);
            dsrc >> token;
            double cap = convertString<double>(token);
            dsrc >> token;
            double throw_away = convertString<double>(token);
			double init = cap*initial_reservoir;
			dsrc >> token;
            double yearly_inflow = convertString<double>(token);
            Reservoir* hr = new Reservoir(mNr,cap,init, yearly_inflow);
            Reservoirs.push_back(hr);
            cnt++;
         }
         if (token == "#") {
            dsrc.ignore(80,'\n');
         }
         cnt = 0;
         reservoir = false;

         for (size_t i=0; i<Reservoirs.size(); i++) {
            int mNr1 = Reservoirs[i]->getModuleNr();
            for (size_t j=0; j<PowerPlants.size(); j++)
            {
               int mNr2 = PowerPlants[j]->getModuleNr();
               if (mNr1 == mNr2) {
                  cnt++;
                  HydroModule* hm = new HydroModule(cnt,*(Reservoirs[i]),*(PowerPlants[j]));
                  HydroModules.push_back(hm);
                  break;
               }
            }
         }
         cnt = 0;
      }
      else if (connectivity) {
         while (dsrc >> token && token != "#") {
            int from = convertString<int>(token)-1;
            dsrc >> token;
            int to = convertString<int>(token)-1;
            HydroModules[from]->setDownStreamModule(HydroModules[to]);
            HydroModules[to]->setUpStreamModules(HydroModules[from]);
            cnt++;
         }
         if (token == "#") {
            dsrc.ignore(80,'\n');
         }
         connectivity = false;
         cnt = 0;
      }
      //else if (demand) {
      //   while (dsrc >> token && token != "#") {
      //      double dm = convertString<double>(token);
      //      Demand* dmnd = new Demand(dm);
      //      FirmDemand = dmnd;
      //      cnt++;
      //   }
      //   if (token == "#") {
      //      dsrc.ignore(80,'\n');
      //   }
      //   demand = false;
      //   cnt = 0;
      //}
   }
   dsrc.close();
   return stop;
}

bool PowerSystem::establishThermalSystemFromFile(std::ifstream& dsrc)
{
   bool stop = false;
   if (dsrc.fail()) {
      std::cout << "Unable to open data file for reading..."<<std::endl;
      stop = true;
   }
   std::string token;
   int cnt = 0;

   while (dsrc >> token && stop == false) {
      if (token == "#") {
         dsrc.ignore(80,'\n');
      }
      else {
         while (dsrc >> token) {
            int Nr = convertString<int>(token);
            dsrc >> token;
            double Cost = convertString<double>(token);
            dsrc >> token;
            double MaxCap = convertString<double>(token);
            ThermalPlant* tp = new ThermalPlant(Nr,Cost,MaxCap);
            ThermalPlants.push_back(tp);
            cnt++;
         }
      }
   }
   return stop;
}

void PowerSystem::establishThermalSystem(){
	int num_thermal = 70;
	double* cost = new double[num_thermal];
	double* max = new double[num_thermal];

	establishThermalCost(cost, num_thermal);
	establishThermalMax(max, num_thermal);
	for (int i = 0; i < num_thermal; i++){
		ThermalPlant* plant = new ThermalPlant(i+1, cost[i], max[i]);
		ThermalPlants.push_back(plant);
	}
}

void PowerSystem::establishThermalCost(double* cost, int num_thermal){
	for (int i = 0; i < num_thermal - 1; i++){
		cost[i] = i + 1;
	}
	cost[num_thermal - 1] = 100;
}

void PowerSystem::establishThermalMax(double* max, int num_thermal){
	double demand_max = 90;
	for (int i = 0; i < num_thermal - 1; i++){
		max[i] = demand_max/num_thermal;
	}
	max[num_thermal - 1] = 1000;
}

void PowerSystem::printThermalSystem(){
	int num_thermal = ThermalPlants.size();
	std::ofstream log;
	log.open("../../output/thermal_system.txt");
	for (int i = 0; i < num_thermal; i ++){
		log << ThermalPlants[i]->getCost() << " " << ThermalPlants[i]->getMaxCap() << std::endl;
	}
	log.close();
}

void PowerSystem::printHydroSystem(){
	int num_modules = HydroModules.size();
	std::ofstream log;
	log.open("../../output/hydro_system.txt");
	for (int i = 0; i < num_modules; i ++){
		log << HydroModules[i]->getPowerPlant().getEfficiency() << " " << HydroModules[i]->getPowerPlant().getMaxOutflow() <<  " " <<
			HydroModules[i]->getReservoir().getCapacity() << " " << HydroModules[i]->getReservoir().getInitial() << " " << 
			HydroModules[i]->getReservoir().get_yearly_inflow() << std::endl;
	}
	log.close();
}