#ifndef POWERSYSTEM_H
#define POWERSYSTEM_H
#include <iostream>
#include <fstream>
#include <sstream>
#include <stdexcept>
#include <vector>
#include "demand.h"

class PowerPlant;
class Reservoir;
class HydroModule;
class ThermalPlant;

class PowerSystem
{
   private:
      std::vector<PowerPlant*> PowerPlants;
	  std::vector<Reservoir*> Reservoirs;
	  std::vector<HydroModule*> HydroModules;
      std::vector<ThermalPlant*> ThermalPlants;

   public:
      PowerSystem(std::ifstream& hydroData,std::ifstream& thermalData);
	  PowerSystem(std::ifstream& hydroData,std::ifstream& thermalData, double initial_reservoir);
      bool establishHydroSystem(std::ifstream& dsrc);
	  bool establishHydroSystem(std::ifstream& dsrc, double);
      bool establishThermalSystemFromFile(std::ifstream& dsrc);
	  void establishThermalSystem();
	  void establishThermalCost(double* cost, int num_thermal);
	  void establishThermalMax(double* max, int num_thermal);
	  void printThermalSystem();
	  void printHydroSystem();
      std::vector<PowerPlant*> getPowerPlants() const {return PowerPlants;}
      std::vector<Reservoir*> getReservoirs() const {return Reservoirs;}
      std::vector<HydroModule*> getHydroModules() const {return HydroModules;}
      HydroModule* getHydroModule(int i) const {return HydroModules[i];}
      std::vector<ThermalPlant*> getThermalPlants() const {return ThermalPlants;}
};
#endif