#ifndef STAGEDATA_H
#define STAGEDATA_H
#include <vector>
#include <fstream>
#include "cut.h"
#include "demand.h"

struct State;

class StageData{
private:
	int stage;
	int num_branches;
	int num_cuts;
	int num_slaves;
	std::vector<Cut*> cuts;
	int* num_cuts_at_slave;
	Demand* demand;
	StageData* previous; //pointer to previous stage
	State** states; //array of all states at this stage
	std::ofstream log;

public:
	StageData(int in_stage, StageData* in_previous, Demand* in_demand, int in_num_branches, int in_num_slaves)
		: stage(in_stage), previous(in_previous), demand(in_demand), num_branches(in_num_branches), num_slaves(in_num_slaves) {
			num_cuts = 0, states = new State*[num_branches];}

	int getStage(){return stage;}
	State* getState(int branch){return states[branch];}
	int getNumCuts(){return num_cuts;}
	int getNumCutsAtSlave(int rank){return num_cuts_at_slave[rank - 1];}
	int getNumNewCuts(int rank) {return num_cuts - num_cuts_at_slave[rank - 1]; }
	double getDemand(int stage){return demand->getDemand(stage);}
	Cut* getCut(int i){return cuts[i];}
	Cut* getNewCut(int i, int rank);

	void Initialize();
	void set_state(int branch, State* state){states[branch] = state;};
	void addCut(Cut* new_cut){cuts.push_back(new_cut); num_cuts++;}
	void addCutToPreviousStage(Cut* new_cut){previous->addCut(new_cut);}
	void UpdateNumCuts(int rank) {num_cuts_at_slave[rank - 1] = num_cuts; }
	void Reset();
};

#endif