#include "substagedata.h"
#include <fstream>

Substagedata::Substagedata(int in_num_substages)
:num_substages(in_num_substages){

	sub_inflow_profile = new double[num_substages];
	sub_demand_profile = new double[num_substages];

	switch (num_substages) {
	case 7: 
		CreateDemandProfileCase7();
		break;
	case 21: 
		CreateDemandProfileCase21();
		break;
	case 168:
		CreateDemandProfileCase168();
		break;
	default:
		CreateDemandProfileDefault();
	}
	CreateInflowProfileDefault();

	Normalize();
}

void Substagedata::CreateDemandProfileCase7(){
	sub_demand_profile[0] = 1.1;
	sub_demand_profile[1] = 1.3;
	sub_demand_profile[2] = 1.3;
	sub_demand_profile[3] = 1.3;
	sub_demand_profile[4] = 1.0;
	sub_demand_profile[5] = 0.7;
	sub_demand_profile[6] = 0.6;
}

void Substagedata::CreateDemandProfileCase21(){
	double* weekly_profile = new double[7];
	double* daily_profile = new double[3];
		
	weekly_profile[0] = 1.1;
	weekly_profile[1] = 1.3;
	weekly_profile[2] = 1.3;
	weekly_profile[3] = 1.3;
	weekly_profile[4] = 1.0;
	weekly_profile[5] = 0.7;
	weekly_profile[6] = 0.6;

	daily_profile[0] = 0.6;
	daily_profile[1] = 1.3;
	daily_profile[2] = 1.0;

	int pos = 0;

	for (int i = 0; i < 7; i++) {
		for (int j = 0; j < 3; j++) {
			sub_demand_profile[pos] = weekly_profile[i]*daily_profile[j]; 
			pos++;
		}
	}
}

void Substagedata::CreateDemandProfileCase168(){
	double* weekly_profile = new double[7];
	double* daily_profile = new double[24];
		
	weekly_profile[0] = 1.1;
	weekly_profile[1] = 1.3;
	weekly_profile[2] = 1.3;
	weekly_profile[3] = 1.3;
	weekly_profile[4] = 1.0;
	weekly_profile[5] = 0.7;
	weekly_profile[6] = 0.6;

	daily_profile[0] = 0.3; //00.00
	daily_profile[1] = 0.2; //01.00
	daily_profile[2] = 0.1; //02.00
	daily_profile[3] = 0.1; 
	daily_profile[4] = 0.2;
	daily_profile[5] = 0.2;
	daily_profile[6] = 0.3;
	daily_profile[7] = 0.5; //07.00
	daily_profile[8] = 0.8;
	daily_profile[9] = 0.9;
	daily_profile[10] = 1.0;
	daily_profile[11] = 1.0;
	daily_profile[12] = 1.0;
	daily_profile[13] = 1.0;
	daily_profile[14] = 0.9;
	daily_profile[15] = 0.7; //15.00
	daily_profile[16] = 0.9;
	daily_profile[17] = 1.0;
	daily_profile[18] = 0.9; //18.00
	daily_profile[19] = 0.8;
	daily_profile[20] = 0.7;
	daily_profile[21] = 0.6;
	daily_profile[22] = 0.5;
	daily_profile[23] = 0.4; //23.00
	int pos = 0;

	for (int i = 0; i < 7; i++) {
		for (int j = 0; j < 24; j++) {
			sub_demand_profile[pos] = weekly_profile[i]*daily_profile[j]; 
			pos++;
		}
	}
}

void Substagedata::CreateDemandProfileDefault(){
	for (int i = 0; i < num_substages; i++) {
		sub_demand_profile[i] = 1.0;
	}
}

void Substagedata::CreateInflowProfileDefault(){
	for (int i = 0; i < num_substages; i++) {
		sub_inflow_profile[i] = 1.0;
	}
}

void Substagedata::Normalize(){
	double sum_inflow_profile = 0;
	double sum_demand_profile = 0;

	for (int i = 0; i < num_substages; i++) {
		sum_inflow_profile += sub_inflow_profile[i];
		sum_demand_profile += sub_demand_profile[i];
	}

	for (int i = 0; i < num_substages; i++) {
		sub_inflow_profile[i] = sub_inflow_profile[i]/sum_inflow_profile;
		sub_demand_profile[i] = sub_demand_profile[i]/sum_demand_profile;
	}
}

void Substagedata::Print() {
	std::ofstream log("../../output/substagedata.txt");
	for (int i = 0; i < num_substages; i++) {
		log << sub_demand_profile[i] << ";";
	}
	log << std::endl;
	for (int i = 0; i < num_substages; i++) {
		log << sub_inflow_profile[i] << ";";
	}
	log << std::endl;
	log.close();
}

double* Substagedata::get_inflow(double current_inflow){
	double* inflow = new double[num_substages];
	for (int i = 0; i < num_substages; i++) {
		inflow[i] = sub_inflow_profile[i]*current_inflow;
	}
	return inflow;
}

double* Substagedata::get_demand(double current_demand){
	double* demand = new double[num_substages];
	for (int i = 0; i < num_substages; i++) {
		demand[i]= sub_demand_profile[i]*current_demand;
	}
	return demand;
}
